"""Test suite."""


from types import GeneratorType
from unittest import TestCase

from brute import brute


class TestBrute(TestCase):

    def test_brute_length_default(self):
        last_str = ''
        for pw in brute():
            last_str = pw

        self.assertEqual(len(last_str), 3)

    def test_brute_returns_generator(self):
        self.assertIsInstance(brute(), GeneratorType)

    def test_letters_numbers_symbols_whitespace_default(self):
        letters = False
        numbers = False
        symbols = False
        whitespace = True

        for pw in brute():
            if 'a' in pw:
                letters = True
            elif '1' in pw:
                numbers = True
            elif '!' in pw:
                symbols = True
            elif ' ' in pw:
                whitespace = False

        self.assertTrue(letters)
        self.assertTrue(numbers)
        self.assertTrue(symbols)
        self.assertTrue(whitespace)

    def test_disable_letters(self):
        letters = True

        for pw in brute(letters=False):
            if 'a' in pw:
                letters = False

        self.assertTrue(letters)

    def test_disable_numbers(self):
        numbers = True

        for pw in brute(numbers=False):
            if '1' in pw:
                numbers = False

        self.assertTrue(numbers)

    def test_disable_symbols(self):
        symbols = True

        for pw in brute(symbols=False):
            if '!' in pw:
                symbols = False

        self.assertTrue(symbols)

    def test_enable_spaces(self):
        spaces = False

        for pw in brute(spaces=True):
            if ' ' in pw:
                spaces = True

        self.assertTrue(spaces)



"""
    brute
    ~~~~~

    Simple brute forcing in Python.
"""


from itertools import chain, product
from random import sample

# Python 2 and 3 have different string constants, so this makes the library
# Python 3 compatible.
try:
    from string import (
        digits as _numbers,
        letters as _letters,
        punctuation as _symbols,
        whitespace as _spaces,
    )
except ImportError:
    from string import (
        digits as _numbers,
        ascii_letters as _letters,
        punctuation as _symbols,
        whitespace as _spaces,
    )


__version__ = '0.0.1'


def brute(length=3, letters=True, numbers=True, symbols=True, spaces=False):
    """
    Iterate through a sequence of possible strings, efficiently.

    :param int length: The length of the string to iterate through.  We'll
        iterate through all permutations of strings up to this length. Default:
        5.
    :param bool letters: Include letters (upper and lower case)? Default: True.
    :param bool numbers: Include numbers? Default: True.
    :param bool symbols: Include symbols? Default: True.
    :param bool space: Include space characters? Default: False.

    :rtype: generator
    :returns: A generator which iterates through all permutations of the
        specified values.
    """
    choices = ''
    choices += _letters if letters else ''
    choices += _numbers if numbers else ''
    choices += _symbols if symbols else ''
    choices += _spaces if spaces else ''
    choices = ''.join(sample(choices, len(choices)))

    return (
        ''.join(candidate) for candidate in
        chain.from_iterable(
            product(
                choices,
                repeat = i,
            ) for i in range(1, length + 1),
        )
    )



"""Packaging information."""


from subprocess import call
from sys import exit

from setuptools import Command, setup

from brute import (
    __doc__ as description,
    __version__ as version,
)


class TestCommand(Command):

    description = 'run all tests'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run the test suite."""
        exit(call(['py.test', '--cov', 'brute']))


setup(

    # Basic package information:
    name = 'brute',
    version = version,
    py_modules = ['brute'],

    # Packaging options:
    zip_safe = False,
    include_package_data = True,

    # Package dependencies:
    install_requires = [],

    # Test harness:
    cmdclass = {
        'test': TestCommand,
    },

    # Metadata for PyPI:
    author = 'Randall Degges',
    author_email = 'r@rdegges.com',
    license = 'UNLICENSE',
    url = 'https://github.com/rdegges/brute',
    keywords = 'python security bruteforce hash hack',
    description = 'Simple brute forcing in Python.',
    long_description = description,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

)
